@@ -93,6 +93,7 @@ public class JavacTaskImpl extends JavacTask {
         args.getClass();
         context.getClass();
         fileObjects.getClass();
+        this.context.put(JavacTaskImpl.class, this);
     }
 
     JavacTaskImpl(JavacTool tool,
@@ -181,7 +182,6 @@ public class JavacTaskImpl extends JavacTask {
     }
 
     private void beginContext() {
-        context.put(JavacTaskImpl.class, this);
         if (context.get(TaskListener.class) != null)
             context.put(TaskListener.class, (TaskListener)null);
         if (taskListener != null)
@@ -361,9 +361,15 @@ public class JavacTaskImpl extends JavacTask {
 
             ListBuffer<TypeElement> elements = new ListBuffer<TypeElement>();
             for (JCCompilationUnit unit : units) {
-                for (JCTree node : unit.defs)
-                    if (node.getTag() == JCTree.CLASSDEF)
-                        elements.append(((JCTree.JCClassDecl) node).sym);
+                boolean isPkgInfo = unit.sourcefile.isNameCompatible("package-info",
+                                                                     JavaFileObject.Kind.SOURCE);
+                if (isPkgInfo) {
+                    elements.append(unit.packge.package_info);
+                } else {
+                    for (JCTree node : unit.defs)
+                        if (node.getTag() == JCTree.CLASSDEF)
+                            elements.append(((JCTree.JCClassDecl) node).sym);
+                }
             }
             return elements.toList();
         }
@@ -377,7 +383,7 @@ public class JavacTaskImpl extends JavacTask {
         final java.util.List<CompilationUnitTree> toEnter = new java.util.ArrayList ();
         final java.util.List<TypeElement> result = new java.util.ArrayList ();
         for (CompilationUnitTree tree : trees) {
-            final java.util.Collection<TypeElement> te = this.getEnteredElement(tree);
+            final java.util.Collection<TypeElement> te = this.getEnteredElements(tree);
             if (te.isEmpty()) {
                 toEnter.add (tree);
             }
@@ -395,14 +401,16 @@ public class JavacTaskImpl extends JavacTask {
     }
 
 
-    private java.util.Collection<TypeElement> getEnteredElement (final CompilationUnitTree tree) {
-        assert tree != null;
-        final java.util.List<Env<AttrContext>> todo = compiler.todo.toList();
+    private java.util.Collection<TypeElement> getEnteredElements (final CompilationUnitTree tree) {
+        assert tree instanceof JCCompilationUnit;
         final java.util.List<TypeElement> result = new java.util.ArrayList<TypeElement>();
-        for (Env<AttrContext> env : todo) {
-            if (env.toplevel.getSourceFile().toUri().equals (((JCCompilationUnit)tree).getSourceFile().toUri())) {
-                this.notYetEntered.remove (((JCCompilationUnit)tree).getSourceFile());
-                result.add(((JCClassDecl)env.tree).sym);
+        if (((JCCompilationUnit)tree).packge != null) {
+            for (JCTree t : ((JCCompilationUnit)tree).defs) {
+                if (t.getTag() == JCTree.CLASSDEF) {
+                    ClassSymbol sym = ((JCClassDecl)t).sym;
+                    if (sym != null)
+                        result.add(sym);
+                }
             }
         }
         return result;
@@ -489,7 +497,7 @@ public class JavacTaskImpl extends JavacTask {
     public Iterable<? extends JavaFileObject> generate(Iterable<? extends TypeElement> classes) throws IOException {
         final ListBuffer<JavaFileObject> results = new ListBuffer<JavaFileObject>();
         try {
-            analyze(null);  // ensure all classes have been parsed, entered, and analyzed
+            analyze(classes);  // ensure all classes have been parsed, entered, and analyzed
 
             if (classes == null) {
                 compiler.generate(compiler.desugar(genList.toList()), results);
@@ -506,6 +514,7 @@ public class JavacTaskImpl extends JavacTask {
             if (genList.isEmpty()) {
                 compiler.reportDeferredDiagnostics();
                 compiler.log.flush();
+                compiler.repair.flush();
                 endContext();
             }
         }
@@ -538,7 +547,6 @@ public class JavacTaskImpl extends JavacTask {
             this.compiler.todo.clear();
         }
         if (this.genList != null && !this.genList.isEmpty()) {
-            assert genList.size() == 1;
             this.genList.clear();
         }
         endContext();
@@ -577,7 +585,14 @@ public class JavacTaskImpl extends JavacTask {
             List<Env<AttrContext>> defer = List.<Env<AttrContext>>nil();
             while (list.nonEmpty()) {
                 Env<AttrContext> env = list.next();
-                ClassSymbol csym = env.enclClass.sym;
+                ClassSymbol csym;
+                boolean isPkgInfo = env.toplevel.sourcefile.isNameCompatible("package-info",
+                                                                             JavaFileObject.Kind.SOURCE);
+                if (isPkgInfo) {
+                    csym = env.toplevel.packge.package_info;
+                } else {
+                    csym = env.enclClass.sym;
+                }
                 if (csym != null && set.contains(csym.outermostClass()))
                     process(env);
                 else
