@@ -1042,7 +1042,8 @@ public abstract class Symbol implements Element {
                         return super.visitType(te, p);
                     }                    
                 }.scan(enclClass);
-                name.table.loader.loadTreeFor(enclClass);
+                if (!name.table.loader.loadTreeFor(enclClass, true))
+                    name.table.loader.loadParamNames(enclClass);
             }
             return super.getSimpleName();
         }
@@ -1292,9 +1293,15 @@ public abstract class Symbol implements Element {
                         names = names.tail;
                     }
                 } else {
+                    int i = 0;
                     for (Type t : type.getParameterTypes()) {
-                        buf.append(new VarSymbol(PARAMETER, names.head, t, this));
-                        names = names.tail;
+                        if (names.nonEmpty()) {
+                            buf.append(new VarSymbol(PARAMETER, names.head, t, this));
+                            names = names.tail;
+                        } else {
+                            buf.append(new VarSymbol(PARAMETER, name.table.fromString("arg" + i), t, this));
+                        }
+                        i++;
                     }
                 }
                 params = buf.toList();
