@@ -103,6 +103,7 @@ public class Enter extends JCTree.Visitor {
     Lint lint;
     JavaFileManager fileManager;
     private final CancelService cancelService;
+    private final LowMemoryWatch memoryWatch;
     private final LazyTreeLoader treeLoader;
     private final Source source;
 
@@ -127,6 +128,7 @@ public class Enter extends JCTree.Visitor {
         annotate = Annotate.instance(context);
         lint = Lint.instance(context);
         cancelService = CancelService.instance(context);
+        memoryWatch = LowMemoryWatch.instance(context);
         treeLoader = LazyTreeLoader.instance(context);
 
         predefClassDef = make.ClassDef(
@@ -289,8 +291,10 @@ public class Enter extends JCTree.Visitor {
      */
     <T extends JCTree> List<Type> classEnter(List<T> trees, Env<AttrContext> env) {
         ListBuffer<Type> ts = new ListBuffer<Type>();
-        for (List<T> l = trees; l.nonEmpty(); l = l.tail)
+        for (List<T> l = trees; l.nonEmpty(); l = l.tail) {
             ts.append(classEnter(l.head, env));
+            memoryWatch.abortIfMemoryLow();
+        }
         return ts.toList();
     }
 
@@ -301,6 +305,7 @@ public class Enter extends JCTree.Visitor {
                                                              JavaFileObject.Kind.SOURCE);
         if (tree.pid != null) {
             tree.packge = reader.enterPackage(TreeInfo.fullName(tree.pid));
+            PackageAttributer.attrib(tree.pid, tree.packge);
             if (tree.packageAnnotations.nonEmpty()) {
                 if (isPkgInfo) {
                     addEnv = true;
@@ -344,6 +349,29 @@ public class Enter extends JCTree.Visitor {
         result = null;
     }
 
+    private static class PackageAttributer extends TreeScanner {
+
+        private Symbol pkg;
+
+        public static void attrib(JCExpression pid, Symbol pkg) {
+            PackageAttributer pa = new PackageAttributer();
+            pa.pkg = pkg;
+            pa.scan(pid);
+        }
+
+        @Override
+        public void visitIdent(JCIdent that) {
+            that.sym = pkg;
+        }
+
+        @Override
+        public void visitSelect(JCFieldAccess that) {
+            that.sym = pkg;
+            pkg = pkg.owner;
+            super.visitSelect(that);
+        }
+    }
+
     public void visitClassDef(JCClassDecl tree) {
         cancelService.abortIfCanceled();
         Symbol owner = env.info.scope.owner;
@@ -390,6 +418,8 @@ public class Enter extends JCTree.Visitor {
                     tree.sym = (ClassSymbol)result.tsym;
                     Env<AttrContext> localEnv = classEnv(tree, env);
                     typeEnvs.put(tree.sym, localEnv);
+                    tree.sym.completer = memberEnter;
+                    if (!tree.sym.isLocal() && uncompleted != null) uncompleted.append(tree.sym);
                     return;
                 }
                 if (owner.kind == TYP) {
@@ -430,14 +460,12 @@ public class Enter extends JCTree.Visitor {
         }
         tree.sym = c;
 
-        if (reattr) {
-            if (c.kind == ERR && c.type.isErroneous()) {
-                c.flags_field &= ~FROMCLASS;
-                c.kind = TYP;
-                c.type = new ClassType(Type.noType, List.<Type>nil(), c);
-            } else {
-                c.flags_field |= FROMCLASS;
-            }
+        if (c.kind == ERR && c.type.isErroneous()) {
+            c.flags_field &= ~FROMCLASS;
+            c.kind = TYP;
+            c.type = new ClassType(Type.noType, List.<Type>nil(), c);
+        } else if (reattr) {
+            c.flags_field |= FROMCLASS;
         }
 
         // Enter class into `compiled' table and enclosing scope.
@@ -610,6 +638,7 @@ public class Enter extends JCTree.Visitor {
                     else
                         // defer
                         prevUncompleted.append(clazz);
+                    memoryWatch.abortIfMemoryLow();
                 }
 
                 // if there remain any unimported toplevels (these must have
@@ -622,6 +651,7 @@ public class Enter extends JCTree.Visitor {
                             env = topLevelEnv(tree);
                         memberEnter.memberEnter(tree, env);
                         log.useSource(prev);
+                        memoryWatch.abortIfMemoryLow();
                     }
                 }
             }
