@@ -32,6 +32,7 @@ import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
+import java.util.logging.Logger;
 import javax.lang.model.SourceVersion;
 import javax.tools.JavaFileObject;
 import javax.tools.JavaFileManager;
@@ -237,10 +238,11 @@ public class ClassReader extends ClassFile implements Completer {
         annotate = Annotate.instance(context);
         verbose        = options.get("-verbose")        != null;
         checkClassFile = options.get("-checkclassfile") != null;
+        boolean ideMode = options.get("ide") != null;
         Source source = Source.instance(context);
-        allowGenerics    = allowCovRetTypes = source.allowGenerics();
-        allowVarargs     = source.allowVarargs();
-        allowAnnotations = source.allowAnnotations();
+        allowGenerics    = allowCovRetTypes = source.allowGenerics() || ideMode;
+        allowVarargs     = source.allowVarargs() || ideMode;
+        allowAnnotations = source.allowAnnotations() || ideMode;
         saveParameterNames = options.get("save-parameter-names") != null;
         cacheCompletionFailure = options.get("dev") == null;
         preferSource = "source".equals(options.get("-Xprefer"));
@@ -578,6 +580,7 @@ public class ClassReader extends ClassFile implements Completer {
         case 'J':
             sigp++;
             return syms.longType;
+        case 'R':
         case 'L':
             {
                 // int oldsigp = sigp;
@@ -633,9 +636,10 @@ public class ClassReader extends ClassFile implements Completer {
     /** Convert class signature to type, where signature is implicit.
      */
     Type classSigToType() {
-        if (signature[sigp] != 'L')
+        if (signature[sigp] != 'L' && signature[sigp] != 'R')
             throw badClassFile("bad.class.signature",
                                Convert.utf2string(signature, sigp, 10));
+        boolean err = signature[sigp] == 'R';
         sigp++;
         Type outer = Type.noType;
         int startSbp = sbp;
@@ -648,7 +652,9 @@ public class ClassReader extends ClassFile implements Completer {
                 ClassSymbol t = enterClass(names.fromUtf(signatureBuffer,
                                                          startSbp,
                                                          sbp - startSbp));
-                if (outer == Type.noType)
+                if (err)
+                    outer = new ErrorType(t, false);
+                else if (outer == Type.noType)
                     outer = t.erasure(types);
                 else
                     outer = new ClassType(outer, List.<Type>nil(), t);
@@ -930,6 +936,19 @@ public class ClassReader extends ClassFile implements Completer {
             int newbp = bp + attrLen;
             readEnclosingMethodAttr(sym);
             bp = newbp;
+        } else if (attrName == names._org_netbeans_TypeSignature) {
+            sym.type = readType(nextChar());
+        } else if (attrName == names._org_netbeans_ParameterNames) {
+            List<Name> parameterNames = List.nil();
+            int numParams = sym.type.getParameterTypes().length();
+            for (int i = 0; i < numParams; i++)
+                parameterNames = parameterNames.prepend(readName(nextChar()));
+            parameterNames = parameterNames.reverse();
+            ((MethodSymbol)sym).savedParameterNames = parameterNames;
+        } else if (attrName == names._org_netbeans_SourceLevelAnnotations) {
+            attachAnnotations(sym);
+        } else if (attrName == names._org_netbeans_SourceLevelParameterAnnotations) {
+            attachParameterAnnotations(sym);
         } else {
             unrecognized(attrName);
             bp = bp + attrLen;
@@ -945,7 +964,12 @@ public class ClassReader extends ClassFile implements Completer {
         ClassSymbol c = readClassSymbol(nextChar());
         NameAndType nt = (NameAndType)readPool(nextChar());
 
-        MethodSymbol m = findMethod(nt, c.members_field, self.flags());
+        MethodSymbol m = null;
+        if (c.members_field != null) {
+            m = findMethod(nt, c.members_field, self.flags());
+        } else {
+            Logger.getLogger(ClassReader.class.getName()).warning("ClassWriter.readEnclosingMethodAttr:" + (c.completer != null ? "uncompleted" : "") + " symbol [" + c + "] of kind [" + c.kind + "] has null members_field."); //NOI18N
+        }
         if (nt != null && m == null)
             throw badClassFile("bad.enclosing.method", self);
 
@@ -1744,7 +1768,7 @@ public class ClassReader extends ClassFile implements Completer {
             c = defineClass(name, owner);
             c.flatname = flatname;
             classes.put(flatname, c);
-        } else if ((c.name != name || c.owner != owner) && c.owner.kind == PCK) {
+        } else if ((c.name != name || c.owner != owner) && (c.owner.kind & (PCK | TYP)) != 0) {
             // reassign fields of classes that might have been loaded with
             // their flat names.
             c.owner.members().remove(c);
@@ -2080,7 +2104,11 @@ public class ClassReader extends ClassFile implements Completer {
     private boolean isSigOverClass (final JavaFileObject a, final JavaFileObject b) {
         String patha = a.getName().toLowerCase();
         String pathb = b.getName().toLowerCase();
-        return pathb.endsWith(".sig") && patha.endsWith(".class");  //NOI18N
+        if (pathb.endsWith(".sig"))  //NOI18N
+            return patha.endsWith(".class");  //NOI18N
+        else if (pathb.endsWith(".class"))  //NOI18N
+            return patha.endsWith(".class") && b.getLastModified() >= a.getLastModified();  //NOI18N
+        return false;
     }
 
     /** Implement policy to choose to derive information from a source
