@@ -241,7 +241,11 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     /** The flow analyzer.
      */
     protected Flow flow;
-
+    
+    /** The error repairer.
+     */
+    public Repair repair;
+    
     /** The type eraser.
      */
     TransTypes transTypes;
@@ -276,6 +280,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
 
     protected FlowListener flowListener;
+    
+    protected LowMemoryWatch memoryWatch;
 
     /**
      * Annotation processing may require and provide a new instance
@@ -315,6 +321,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         enter = Enter.instance(context);
         todo = Todo.instance(context);
         flowListener = FlowListener.instance(context);
+        memoryWatch = LowMemoryWatch.instance(context);
 
         fileManager = context.get(JavaFileManager.class);
         parserFactory = Parser.Factory.instance(context);
@@ -333,6 +340,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         chk = Check.instance(context);
         gen = Gen.instance(context);
         flow = Flow.instance(context);
+        repair = Repair.instance(context);
         transTypes = TransTypes.instance(context);
         lower = Lower.instance(context);
         annotate = Annotate.instance(context);
@@ -821,8 +829,10 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
         //parse all files
         ListBuffer<JCCompilationUnit> trees = lb();
-        for (JavaFileObject fileObject : fileObjects)
+        for (JavaFileObject fileObject : fileObjects) {
             trees.append(parse(fileObject));
+            memoryWatch.abortIfMemoryLow();
+        }
         return trees.toList();
     }
 
@@ -1036,8 +1046,10 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     public List<Env<AttrContext>> attribute(ListBuffer<Env<AttrContext>> envs) {
         ListBuffer<Env<AttrContext>> results = lb();
-        while (envs.nonEmpty())
+        while (envs.nonEmpty()) {
             results.append(attribute(envs.next()));
+            memoryWatch.abortIfMemoryLow();
+        }
         return results.toList();
     }
 
@@ -1080,8 +1092,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         ListBuffer<Env<AttrContext>> results = lb();
         for (List<Env<AttrContext>> l = envs; l.nonEmpty(); l = l.tail) {
             flow(l.head, results);
+            memoryWatch.abortIfMemoryLow();
         }
-        return stopIfError(results);
+        return results.toList();
     }
 
     /**
@@ -1090,7 +1103,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     public List<Env<AttrContext>> flow(Env<AttrContext> env) {
         ListBuffer<Env<AttrContext>> results = lb();
         flow(env, results);
-        return stopIfError(results);
+        return results.toList();
     }
 
     /**
@@ -1098,9 +1111,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     protected void flow(Env<AttrContext> env, ListBuffer<Env<AttrContext>> results) {
         try {
-            if (errorCount() > 0)
-                return;
-
             if (relax || deferredSugar.contains(env)) {
                 results.append(env);
                 return;
@@ -1122,8 +1132,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 if (flowListener != null) {
                     flowListener.flowFinished (env);
                 }
-                if (errorCount() > 0)
-                    return;
 
                 results.append(env);
             }
@@ -1139,6 +1147,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         }
     }
 
+    public boolean doRepair = true; // Allows for switching off repair. For test purposes only.
+
     /**
      * Prepare attributed parse trees, in conjunction with their attribution contexts,
      * for source or code generation.
@@ -1147,9 +1157,11 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     public List<Pair<Env<AttrContext>, JCClassDecl>> desugar(List<Env<AttrContext>> envs) {
         ListBuffer<Pair<Env<AttrContext>, JCClassDecl>> results = lb();
-        for (List<Env<AttrContext>> l = envs; l.nonEmpty(); l = l.tail)
+        for (List<Env<AttrContext>> l = envs; l.nonEmpty(); l = l.tail) {
             desugar(l.head, results);
-        return stopIfError(results);
+            memoryWatch.abortIfMemoryLow();
+        }
+        return results.toList();
     }
 
     /**
@@ -1159,9 +1171,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      * The preparation stops as soon as an error is found.
      */
     protected void desugar(Env<AttrContext> env, ListBuffer<Pair<Env<AttrContext>, JCClassDecl>> results) {
-        if (errorCount() > 0)
-            return;
-
         if (implicitSourcePolicy == ImplicitSourcePolicy.NONE
                 && !inputFiles.contains(env.toplevel.sourcefile)) {
             return;
@@ -1199,6 +1208,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 return;
             }
 
+            if (doRepair)
+                env.tree = repair.translateTopLevelClass(env, env.tree, localMake);
+
             if (stubOutput) {
                 //emit stub Java source file, only for compilation
                 //units enumerated explicitly on the command line
@@ -1214,9 +1226,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
             env.tree = transTypes.translateTopLevelClass(env.tree, localMake);
 
-            if (errorCount() != 0)
-                return;
-
             if (sourceOutput) {
                 //emit standard Java source file, only for compilation
                 //units enumerated explicitly on the command line
@@ -1231,9 +1240,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             //translate out inner classes
             List<JCTree> cdefs = lower.translateTopLevelClass(env, env.tree, localMake);
 
-            if (errorCount() != 0)
-                return;
-
             //generate code for each class
             for (List<JCTree> l = cdefs; l.nonEmpty(); l = l.tail) {
                 JCClassDecl cdef = (JCClassDecl)l.head;
@@ -1334,6 +1340,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 TaskEvent e = new TaskEvent(TaskEvent.Kind.GENERATE, env.toplevel, cdef.sym);
                 taskListener.finished(e);
             }
+            
+            memoryWatch.abortIfMemoryLow();
         }
     }
 
