@@ -500,7 +500,7 @@ public class TreeInfo {
                 else super.visitVarDef(that);
             }
             public void visitTypeParameter(JCTypeParameter that) {
-                if (that.type.tsym == sym) result = that;
+                if (that.type != null && that.type.tsym == sym) result = that;
                 else super.visitTypeParameter(that);
             }
         }
@@ -666,7 +666,8 @@ public class TreeInfo {
         case JCTree.TYPEAPPLY:
             return symbolFor(((JCTypeApply) node).clazz);
         case JCTree.TYPEPARAMETER:
-            return ((JCTypeParameter) node).type.tsym;
+            if (((JCTypeParameter) node).type != null)
+                return ((JCTypeParameter) node).type.tsym;
         default:
             return null;
         }
