@@ -615,7 +615,9 @@ public class TreeMaker implements JCTree.Factory {
             break;
         case WILDCARD: {
             WildcardType a = ((WildcardType) t);
-            tp = Wildcard(TypeBoundKind(a.kind), a.type != syms.objectType ? Type(a.type) : null);
+            tp = a.type != syms.objectType
+                    ? Wildcard(TypeBoundKind(a.kind), Type(a.type))
+                    : Wildcard(TypeBoundKind(BoundKind.UNBOUND), null);
             break;
         }
         case CLASS:
@@ -831,6 +833,11 @@ public class TreeMaker implements JCTree.Factory {
     /** Construct an assignment from a variable symbol and a right hand side.
      */
     public JCStatement Assignment(Symbol v, JCExpression rhs) {
+        if (rhs.getTag() == JCTree.ERRONEOUS) {
+            JCErroneous err = (JCErroneous)rhs;
+            if (err.errs.head != null && err.errs.head.getTag() == JCTree.THROW)
+                return (JCThrow)err.errs.head;
+        }
         return Exec(Assign(Ident(v), rhs).setType(v.type));
     }
 
